var consoleFileLogger = require("./sl-logger.js").consoleFileLogger;
var fs = require('fs');
var parser = require("gitignore-parser");

function SlIgnoreWrapper(slIgnoreFilePath, outputPath, logger){
    this.log = this._getCreateLogger(logger);
    this.slIgnoreFilePath = slIgnoreFilePath;
    this.outputPath = outputPath;
    this.parsed = this._getParser();
}


SlIgnoreWrapper.prototype._getParser = function() {
    var pattern = this._getIgnorePattern();
    try {
        var compiled = parser.compile(pattern); //may throw if the file is invalid/corrupted
        this.log.info('Using .slignore file ' + this.slIgnoreFilePath);
        return compiled;
    } catch (e) {
        this.log.error('Not using corrupted .slignore file ' + this.slIgnoreFilePath + ". Using default behavior.");
        pattern = this._getDefaultPattern(pattern, this.outputPath);
        return parser.compile(pattern); //should not throw again
    }
}

SlIgnoreWrapper.prototype._getIgnorePattern = function (){
    var pattern;
    try {
        pattern = fs.readFileSync(this.slIgnoreFilePath, 'utf8');
        pattern = this._addExcludedPath(pattern, this.outputPath);
    }catch(err){
        this.log.info("Failed to read from: " + this.slIgnoreFilePath + ". Using default behavior.")
        pattern = this._getDefaultPattern(pattern, this.outputPath);
    }
    return pattern
}

SlIgnoreWrapper.prototype._getCreateLogger =function (logger) {
    var log = logger ? logger.child({
        methodName: "getSlIgnore"
    }) : consoleFileLogger("getSlIgnore");
    return log;
}

SlIgnoreWrapper.prototype._addExcludedPath = function(pattern, excludedFolder) {
    if (!excludedFolder) return pattern;
    if (excludedFolder && excludedFolder[excludedFolder.length - 1] != '/') { //Ensure the excludedFolder ends with / to comply with gitignore
        excludedFolder += '/';
    }
    pattern = pattern + "\n" + excludedFolder;
    return pattern;
}

SlIgnoreWrapper.prototype._getDefaultPattern = function() {
    // default pattern - all js files under test directory and it's sub directories.
    var pattern = "";
    pattern = this._addExcludedPath(pattern, this.outputPath);
    return pattern;
}

SlIgnoreWrapper.prototype.denies = function(input){
    return this.parsed.denies(input);
}

SlIgnoreWrapper.prototype.accepts= function(input){
    return this.parsed.accepts(input);
}


module.exports = SlIgnoreWrapper;



